import subprocess
from dataclasses import dataclass
from typing import Optional, Tuple

import questionary

from . import prompt
from .info import LotsInfo


@dataclass
class BuyInfo(prompt.Tradeinfo):
    base_cur: str


def val_buy_qtty(answer: str):
    try:
        answer_float = float(answer)
    except ValueError:
        return "Invalid number"

    if answer_float <= 0:
        return "Quantity should be positive"

    return True


class PromptBuy(prompt.Prompt):
    def __init__(
        self,
        file: Tuple[str, ...],
        avg_cost: bool,
        check: bool,
        no_desc: Optional[str] = None,
    ) -> None:
        super().__init__(file, avg_cost, check, no_desc)
        all_commodities_txt = self.run_hledger_no_query_desc("commodities")
        self.all_commodities = [
            com for com in all_commodities_txt.split("\n") if com != ""
        ]

        print(
            """
            Append a new purchase of a commodity.
            Use format format "y.ticker" to automatically download prices.
            Search on yahoo finance website the ticker for the commodity.\n"""
        )
        print(self.initial_info)
        self.info = self.get_info()
        self.last_purchase = self.get_last_purchase(self.info)

    def get_info(self):
        commodity = prompt.ask_commodities_text(self.all_commodities)
        info_not_found = LotsInfo(
            comm=commodity,
            cur="",
            qtty="0",
            amount="0",
            avg_cost="0",
            mkt_price=None,
            mkt_date=None,
            mkt_amount=None,
            mkt_profit=None,
            xirr=None,
        )
        info = next(
            (info for info in self.infos if info["comm"] == commodity), info_not_found
        )
        return info

    def ask_base_cur_text(self):
        if self.info["cur"] == "":
            answer: str = prompt.custom_autocomplete(
                "Base Currency", self.all_commodities
            ).ask()
        else:
            answer = self.info["cur"]
        return answer

    def ask_buy_qtty(self, info: LotsInfo):
        available = float(info["qtty"])

        answer_str: str = questionary.text(
            f"Quantity (available {available})",
            validate=val_buy_qtty,
            instruction="",
        ).ask()
        return answer_str

    def ask_commodity_account(self):
        accts_txt = self.run_hledger("accounts")
        accts = [acct for acct in accts_txt.split("\n") if acct != ""]
        answer: str = prompt.custom_autocomplete("Commodity Account", accts).ask()
        return answer

    def prompt(self):
        commodity = self.info["comm"]
        base_cur = self.ask_base_cur_text()
        sell_date = self.ask_date(self.last_purchase)
        qtty = float(self.ask_buy_qtty(self.info))
        price_str = self.ask_price(self.info)

        if price_str == "":
            value_str = self.ask_total(qtty, self.info)
            value = float(value_str)
            price = value / qtty
        else:
            price = float(price_str)
            value = qtty * price

        cash_acct = self.ask_cash_account()

        commodity_acct = self.ask_commodity_account()

        result = BuyInfo(
            date=sell_date,
            quantity=qtty,
            commodity=commodity,
            base_cur=base_cur,
            cash_account=cash_acct,
            commodity_account=commodity_acct,
            price=price,
            value=value,
        )
        return result

    def get_hl_txn(self):
        buy = self.prompt()

        txn_raw = f"""
{buy.date} Buy {buy.commodity}
    {buy.commodity_account}    {buy.quantity} \"{buy.commodity}\" @ {buy.price} \"{buy.base_cur}\"
    {buy.cash_account}
"""

        comm = ["hledger", "-f-", "print", "--explicit"]
        txn_proc = subprocess.run(
            comm,
            input=txn_raw.encode(),
            capture_output=True,
        )
        if txn_proc.returncode != 0:
            err = txn_proc.stderr.decode("utf8")
            raise prompt.PromptError(err)

        txn_print: str = txn_proc.stdout.decode("utf8")
        return txn_print



import sys
from dataclasses import dataclass
from datetime import date, datetime, timedelta
from pathlib import Path
from typing import Dict, List, Tuple

import yfinance as yf
from requests.exceptions import HTTPError
from requests_cache import CachedSession

from .commodity_tag import CommodityDirective, CommodityTag
from .hl import hledger2txn
from .info import get_last_price
from .lib import get_files_comm


@dataclass
class Price:
    name: str
    date: date
    price: float
    cur: str


class YahooPrices:
    TAG = "yahoo_ticker"

    def __init__(self, files: Tuple[str, ...]) -> None:
        self.files = files
        self.files_comm = get_files_comm(files)

        self.session_path = Path.home() / "yfinance.cache"
        self.session = CachedSession(str(self.session_path))
        self.today = datetime.today()
        yesterday = self.today - timedelta(days=1)
        self.yesterday_str = yesterday.strftime("%Y-%m-%d")

        commodity_directive = CommodityDirective(self.files)
        self.commodities = commodity_directive.get_commodity_tag(self.TAG)

    def get_start_date(self, commodity: CommodityTag):
        txns = hledger2txn(self.files, commodity["commodity"])
        qtty = sum(txn.qtty for txn in txns)
        if qtty == 0:
            print(
                f"; stderr: No transaction for {commodity['commodity']}. Not downloading ",
                file=sys.stderr,
            )
            return

        first_date_str = txns[0].date
        first_date = datetime.strptime(first_date_str, "%Y-%m-%d").date()
        last_market_date = get_last_price(self.files_comm, commodity["commodity"])[0]

        if not last_market_date:
            last_date = first_date
        elif last_market_date < first_date:
            last_date = first_date
        else:
            last_date = last_market_date

        start_date = last_date + timedelta(days=1)
        past = date.today() - start_date
        if past.days < 1:
            print(f"; stderr: No new data for {commodity}", file=sys.stderr)
            return

        return start_date

    def prices2hledger(self, prices: List[Price]):
        prices_list = [
            f"P {price.date.strftime('%Y-%m-%d')} \"{price.name}\" {price.price} {price.cur}"
            for price in prices
            if price
        ]
        prices_str = "\n".join(prices_list)
        return prices_str

    def get_prices(
        self,
        commodity: CommodityTag,
        start_date: str,
    ) -> List[Price]:
        ticker = yf.Ticker(commodity["value"], session=self.session)

        try:
            info: Dict[str, str] = ticker.info
        except HTTPError:
            print(f"; stderr: Can't download commodity {commodity}", file=sys.stderr)
            return []

        if not start_date:
            return []

        df = ticker.history(start=start_date, end=self.yesterday_str, raise_errors=True)

        prices = [
            Price(
                commodity["commodity"],
                row[0].to_pydatetime().date(),  # type:ignore
                row[1]["Close"],  # type: ignore
                info["currency"],
            )
            for row in df.iterrows()
        ]
        return prices

    def get_commodity_prices(self, commodity: CommodityTag):
        start_date = self.get_start_date(commodity)
        if not start_date:
            return

        start_date_str = start_date.strftime("%Y-%m-%d")
        try:
            prices = [
                price for price in self.get_prices(commodity, start_date_str) if price
            ]
            return prices
        except HTTPError:
            print(f"; stderr: {commodity['value']} not found", file=sys.stderr)
        except Exception:
            print(
                f"; stderr: Nothing downloaded for {commodity['value']} between {start_date} and {self.yesterday_str}",
                file=sys.stderr,
            )

    def print_prices(self):
        if len(self.commodities) == 0:
            print(
                f"\n\n; stderr: No commodities directives with tag {self.TAG}",
                file=sys.stderr,
            )
            return

        for commodity in self.commodities:
            prices = self.get_commodity_prices(commodity)
            if prices:
                print("\n")
                hledger_prices = self.prices2hledger(prices)
                print(hledger_prices)



import subprocess
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional, Tuple

import questionary
from prompt_toolkit.shortcuts import CompleteStyle

from .avg_info import AllAvgInfo
from .fifo_info import AllFifoInfo
from .info import LotsInfo
from .lib import get_files_comm


class PromptError(BaseException):
    def __init__(self, message: str) -> None:
        self.message = message
        super().__init__(self.message)


@dataclass
class Tradeinfo:
    date: str
    quantity: float
    commodity: str
    cash_account: str
    commodity_account: str
    price: float
    value: float


def custom_autocomplete(name: str, choices: List[str]):
    question = questionary.autocomplete(
        f"{name} (TAB to autocomplete)",
        choices=choices,
        ignore_case=True,
        match_middle=True,
        style=questionary.Style([("answer", "fg:#f71b07")]),
        complete_style=CompleteStyle.MULTI_COLUMN,
    )
    return question


def get_append_file(default_file: str):
    confirm = questionary.confirm(
        "Add sale transaction to a journal", default=False, auto_enter=True
    ).ask()

    if confirm:
        file_append: str = questionary.path(
            "File to append transaction", default=default_file
        ).ask()
        comm = ["hledger", "-f", file_append, "check"]
        subprocess.run(comm, check=True)
        return file_append


def select_commodities_text(commodities: List[str]):
    answer = questionary.select(
        "Commodity",
        choices=commodities,
        use_shortcuts=True,
    ).ask()
    return answer


def ask_commodities_text(commodities: List[str]):
    answer: str = custom_autocomplete("Commodity", commodities).ask()
    return answer


def val_date(date: str):
    try:
        datetime.strptime(date, "%Y-%m-%d")
        return True
    except ValueError:
        return "Invalid date format"


def val_sell_qtty(answer: str, available: float):
    try:
        answer_float = float(answer)
    except ValueError:
        return "Invalid number"

    if answer_float <= 0:
        return "Quantity should be positive"

    if answer_float > available:
        return "Quantity should be less than available"

    return True


def val_price(answer: str):
    if answer == "":
        return True

    try:
        answer_float = float(answer)
    except ValueError:
        return "Invalid number"

    if answer_float < 0:
        return "Price should be positive"

    return True


def val_total(answer: str):
    try:
        answer_float = float(answer)
    except ValueError:
        return "Invalid number"

    if answer_float < 0:
        return "Amount should be positive"

    return True


class Prompt:
    def __init__(
        self,
        file: Tuple[str, ...],
        avg_cost: bool,
        check: bool,
        no_desc: Optional[str] = None,
    ) -> None:
        self.file = file
        self.check = check
        self.no_desc = no_desc
        self.avg_cost = avg_cost

        self.files_comm = get_files_comm(file)
        self.infos = self.get_infos()
        self.commodities = [info["comm"] for info in self.get_infos()]

    def run_hledger(self, *comm: str):
        command = ["hledger", *self.files_comm, *comm]

        if self.no_desc and self.no_desc != "":
            command = [*command, f"not:desc:{self.no_desc}"]

        proc = subprocess.run(command, capture_output=True)
        if proc.returncode != 0:
            raise subprocess.SubprocessError(proc.stderr.decode("utf8"))

        result = proc.stdout.decode("utf8")
        return result

    def run_hledger_no_query_desc(self, *comm: str):
        command = ["hledger", *self.files_comm, *comm]
        proc = subprocess.run(command, capture_output=True)
        if proc.returncode != 0:
            raise subprocess.SubprocessError(proc.stderr.decode("utf8"))

        result = proc.stdout.decode("utf8")
        return result

    def get_infos(self):
        if self.avg_cost:
            infos = AllAvgInfo(self.file, self.no_desc or "", self.check)
        else:
            infos = AllFifoInfo(self.file, self.no_desc or "", self.check)

        valid_infos = [info for info in infos.infos if float(info["qtty"]) > 0]
        return valid_infos

    def get_last_purchase(self, info: LotsInfo):
        commodity = info["comm"]
        reg = self.run_hledger("reg", f"cur:{commodity}", "amt:>0")
        rows_list = [row for row in reg.split("\n") if row != ""]

        if len(rows_list) > 0:
            last_date = rows_list[-1][0:10]
            return last_date

    def get_append_file(self):
        default_file = self.file[0]

        confirm = questionary.confirm(
            "Add sale transaction to a journal", default=False, auto_enter=True
        ).ask()

        if confirm:
            file_append: str = questionary.path(
                "File to append transaction", default=default_file
            ).ask()
            comm = ["hledger", "-f", file_append, "check"]
            subprocess.run(comm, check=True)
            return file_append

    def ask_date(self, last_purchase: Optional[str]):
        last_purchase = last_purchase

        answer: str = questionary.text(
            f"Date YYYY-MM-DD",
            validate=val_date,
            instruction=f"(Last Purchase: {last_purchase})",
        ).ask()
        return answer

    def ask_sell_qtty(self, info: LotsInfo):
        available = float(info["qtty"])

        answer_str: str = questionary.text(
            f"Quantity (available {available})",
            validate=lambda answer: val_sell_qtty(answer, available),
            instruction="",
        ).ask()
        return answer_str

    def ask_price(self, info: LotsInfo):
        cost_str = info["avg_cost"].replace(",", ".")
        cost = float(cost_str)

        answer_str: str = questionary.text(
            f"Price (avg cost: {cost})",
            validate=val_price,
            instruction="Empty to input total amount",
        ).ask()
        return answer_str

    def ask_total(self, qtty: float, info: LotsInfo):
        available = qtty * float(info["avg_cost"])
        available_str = f"{available:,.2f}"

        answer_str: str = questionary.text(
            f"Total (available {available_str})",
            validate=val_total,
            instruction="Amount received",
        ).ask()
        return answer_str

    def ask_cash_account(self):
        accts_txt = self.run_hledger("accounts")
        accts = [acct for acct in accts_txt.split("\n") if acct != ""]
        answer: str = custom_autocomplete("Cash Account", accts).ask()
        return answer

    def ask_revenue_account(self):
        accts_txt = self.run_hledger("accounts")
        accts = [acct for acct in accts_txt.split("\n") if acct != ""]
        answer: str = custom_autocomplete("Revenue Account", accts).ask()
        return answer

    @property
    def initial_info(self):
        cost_method_text = "Average Cost" if self.avg_cost else "Fifo"
        no_desc_text = self.no_desc or "None"
        check_text = "Checking" if self.check else "Not Checking"

        files = [f if f != "-" else "stdin" for f in self.file]
        files_text = " ".join(files)

        result = f"""
Files              : {files_text}
Cost Method        : {cost_method_text} - {check_text}
Remove description : {no_desc_text}
"""
        return result



import re
from typing import List, Tuple, TypedDict


class CommodityTag(TypedDict):
    commodity: str
    value: str


def get_commodity_name(commodity_directive: str):
    commodity = re.sub(r"(\d[,\d.]*\d)|( )|\'|\"", "", commodity_directive)
    return commodity


def get_comment_tag_value(comment: str, tag: str) -> str:
    search = re.search(f"{tag}:\\s?(\\S+)", comment)
    if search and len(search.groups()) == 1:
        return search.group(1)
    else:
        return ""


class CommodityDirective:
    def __init__(self, files: Tuple[str, ...]):
        self.files = files
        self.rows = self.get_commodities_rows()

    def get_commodities_rows(self) -> List[str]:
        rows = []
        for file in self.files:
            with open(file, "r") as f:
                for row in f:
                    if row.startswith("commodity"):
                        rows.append(row)
        return rows

    def get_commodity_tag(self, tag: str):
        regex = re.compile(r"commodity (.+)(;.+)")
        searches = (regex.search(row) for row in self.rows)
        commented_search = (
            search for search in searches if search and len(search.groups()) == 2
        )
        commodities = (
            CommodityTag(
                commodity=get_commodity_name(comment.group(1)),
                value=get_comment_tag_value(comment.group(2), tag),
            )
            for comment in commented_search
        )
        commodities = [
            commodity for commodity in commodities if "" not in commodity.values()
        ]

        return commodities



from typing import Tuple, TypedDict

import rich_click as click

from .avg_info import AllAvgInfo, AvgInfo
from .fifo_info import AllFifoInfo, FifoInfo
from .info import AllInfo
from .lib import get_default_file, get_file_from_stdin, get_files_comm
from .options import Options, get_options
from .prices_yahoo import YahooPrices
from .prompt import get_append_file
from .prompt_buy import PromptBuy
from .prompt_sell import PromptSell


class Obj(TypedDict):
    file: Tuple[str, ...]
    opt: Options


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])

click.rich_click.USE_MARKDOWN = True
click.rich_click.SHOW_ARGUMENTS = True
click.rich_click.GROUP_ARGUMENTS_OPTIONS = True
click.rich_click.MAX_WIDTH = 80
click.rich_click.SHOW_METAVARS_COLUMN = False
click.rich_click.APPEND_METAVARS_HELP = True
click.rich_click.STYLE_ERRORS_SUGGESTION = "magenta italic"
click.rich_click.ERRORS_SUGGESTION = (
    "Try running the '--help' flag for more information."
)
click.rich_click.ERRORS_EPILOGUE = "To find out more, visit [link=https://github.com/edkedk99/hledger-lots]https://github.com/edkedk99/hledger-lots[/link]"
click.rich_click.STYLE_OPTIONS_TABLE_LEADING = 1
click.rich_click.STYLE_OPTIONS_TABLE_BOX = "SIMPLE"
click.rich_click.STYLE_OPTIONS_PANEL_BORDER = "dim"  # Possibly conceal


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option(
    "-f",
    "--file",
    required=True,
    default=lambda: get_default_file(),
    multiple=True,
    help="Inform the journal file path. If \"-\", read from stdin. Without this flag read from $LEDGER_FILE or ~/.hledger.journal in this order  or '-f-'.",
)
@click.pass_obj
@click.version_option()
def cli(obj, file: Tuple[str, ...]):
    """
    Commands to apply FIFO(first-in-first-out) or AVERAGE COST accounting principles without manual management of lots. Useful for transactions involving buying and selling foreign currencies or stocks.

    To find out more, visit [https://github.com/edkedk99/hledger-lots](https://github.com/edkedk99/hledger-lots)
    """

    if file[0] == "-":
        stdin_file = (get_file_from_stdin(),)
        opt = get_options(stdin_file)
        obj["file"] = stdin_file
    else:
        opt = get_options(file)
        obj["file"] = file

    obj["opt"] = opt


@click.command()
@click.pass_obj
def buy(obj: Obj):
    """
    Create a purchase transaction for a commodity by answering some prompts that tries to avoid errors with validation and using current journal data to filter possible answers and give informations that guides the user thru the process.\r

    ### Transaction postings

    - First posting: Negative amount on the cash account where the money was used to pay for the commodity

    - Second Posting: Positive amount of the commodity being bought with its cost using \"@\" symbol
    """

    file = obj["file"]
    opt = obj["opt"]
    prompt_buy = PromptBuy(file, opt.avg_cost, opt.check, opt.no_desc)
    txn_print = prompt_buy.get_hl_txn()
    click.echo("\n" + txn_print)

    append_file = get_append_file(file[0])
    if append_file:
        with open(append_file, "a") as f:
            f.write("\n" + txn_print)
    else:
        click.echo("\n" + "Transaction not saved.")

    commodity = prompt_buy.info["comm"]
    if opt.avg_cost:
        info = AvgInfo(file, commodity, opt.check)
    else:
        info = FifoInfo(file, commodity, opt.check)

    click.echo(info.table)
    click.echo(info.info_txt)


@click.command()
@click.pass_obj
def sell(obj: Obj):
    """
    Create a transaction with automatic FIFO or AVERAGE COST for a commodity by answering some prompts that tries to avoid errors with validation and using current journal data to filter possible answers give informations that guides the user thru the process.\r

    > This command also add transaction's comment with some indicators. See an example on "Output examples" page of the docs.

    ### Transaction postings

    - First posting: Positive amount on the 'base-currency' in the account that receives the product of the sale.

    - Multiple lot postings: Each posting represents a lot you are selling for the cost price on purchasing date, according to FIFO accounting principles or one postings in case of AVERAGE COST method.

    - Revenue posting: posting that represent Capital Gain or Loss as the difference between the total cost and the amount received on the base-currency.
    """
    file = obj["file"]
    opt = obj["opt"]
    prompt_sell = PromptSell(file, opt.avg_cost, opt.check, opt.no_desc)

    txn_print = prompt_sell.get_hl_txn()
    click.echo("\n" + txn_print)

    append_file = get_append_file(file[0])
    if append_file:
        with open(append_file, "a") as f:
            f.write("\n" + txn_print)
    else:
        click.echo("\n" + "Transaction not saved.")

    commodity = prompt_sell.info["comm"]
    if opt.avg_cost:
        info = AvgInfo(file, commodity, opt.check)
    else:
        info = FifoInfo(file, commodity, opt.check)

    click.echo(info.table)
    click.echo(info.info_txt)

    if commodity.startswith("y."):
        files_comm_str = " ".join(get_files_comm(file))
        click.echo(
            f"To update prices from Yahoo finance, run:\n\n hledger-lots {files_comm_str} view -c {commodity} -p {file[0]}"
        )


@click.command()
@click.argument("commodity", type=click.STRING, required=True)
@click.pass_obj
def view(obj: Obj, commodity: str):
    """
    Report lots for a commodity.\r

    Show a report with lots for a commodity considering eventual past sale using FIFO or AVERAGE COST accounting principles.

    Also show some indicators about the lots and performance if there is prices in the journal after the last purchase. See the docs for details
    """
    file = obj["file"]
    opt = obj["opt"]

    if opt.avg_cost:
        info = AvgInfo(file, commodity, opt.check, opt.no_desc)
    else:
        info = FifoInfo(file, commodity, opt.check, opt.no_desc)

    click.echo(info.table)
    click.echo(info.info_txt)


@click.command(name="list")
@click.option(
    "-o",
    "--output-format",
    type=click.Choice(["plain", "pretty", "csv"]),
    default="plain",
    help="Format to output the report",
)
@click.pass_obj
def list_commodities(obj: Obj, output_format: str):
    """
    List indicators for all your commodities in a tabular format sorted from higher to lower **XIRR**. It is advised to use full-screen of the terminal. See the docs for a list of indicators and output examples.

    It can output in three formats: *plain, pretty and csv*.
    """

    file = obj["file"]
    opt = obj["opt"]
    lots_info = AllInfo(file, opt.no_desc)

    lots_info = (
        AllAvgInfo(file, opt.no_desc, opt.check)
        if opt.avg_cost
        else AllFifoInfo(file, opt.no_desc, opt.check)
    )

    if output_format == "pretty":
        table = lots_info.infos_table("mixed_grid")
    elif output_format == "csv":
        infos_io = lots_info.infos_csv()
        table = infos_io.read()
    else:
        table = lots_info.infos_table("plain")

    click.echo(table)


@click.command()
@click.pass_obj
def prices(obj: Obj):
    """
    Download market prices from Yahoo Finance and print as **price directives**. Use *BASH* redirection to append to the journal or copy/paste the data.

    ### Setup
    Add a \"yahoo_ticker\" tag to the *commodity directive* with the value of the ticker in Yahoo Finance to download prices

    ### Example

    ```text
    commodity AAPL       ; yahoo_ticker:AAPL
    commodity \"PETR4\"    ; yahoo_ticker:PETR4.SA
    commodity BTC        ; yahoo_ticker:BTC-USD
    ```
    """

    file = obj["file"]

    yahoo_prices = YahooPrices(file)
    yahoo_prices.print_prices()


cli.add_command(buy)
cli.add_command(sell)
cli.add_command(view)
cli.add_command(list_commodities)
cli.add_command(prices)



from dataclasses import dataclass
from typing import Optional, Tuple

import questionary

from . import avg, fifo, prompt
from .hl import hledger2txn
from .info import LotsInfo


@dataclass
class SellInfo(prompt.Tradeinfo):
    revenue_account: str


class PromptSell(prompt.Prompt):
    def __init__(
        self,
        file: Tuple[str, ...],
        avg_cost: bool,
        check: bool,
        no_desc: Optional[str] = None,
    ) -> None:
        super().__init__(file, avg_cost, check, no_desc)

        print(self.initial_info)
        self.info = self.get_info()
        self.last_purchase = self.get_last_purchase(self.info)

    def get_info(self):
        commodity = prompt.select_commodities_text(self.commodities)
        info = next(info for info in self.infos if info["comm"] == commodity)
        return info

    def ask_commodity_account(self, info: LotsInfo):
        commodity = info["comm"]
        accts_txt = self.run_hledger("accounts", "note:Buy", f"cur:{commodity}")
        accts = [acct for acct in accts_txt.split("\n") if acct != ""]

        answer: str = questionary.select(
            "Commodity Account",
            choices=accts,
            use_shortcuts=True,
        ).ask()
        return answer

    def prompt(self):
        commodity = self.info["comm"]
        sell_date = self.ask_date(self.last_purchase)
        qtty = float(self.ask_sell_qtty(self.info))
        price_str = self.ask_price(self.info)

        if price_str == "":
            value_str = self.ask_total(qtty, self.info)
            value = float(value_str)
            price = value / qtty
        else:
            price = float(price_str)
            value = qtty * price

        cash_acct = self.ask_cash_account()

        if self.avg_cost:
            commodity_acct = self.ask_commodity_account(self.info)
        else:
            commodity_acct = ""

        revenue_acct = self.ask_revenue_account()

        result = SellInfo(
            date=sell_date,
            quantity=qtty,
            commodity=commodity,
            cash_account=cash_acct,
            revenue_account=revenue_acct,
            commodity_account=commodity_acct,
            price=price,
            value=value,
        )

        return result

    def get_hl_txn(self):
        sell = self.prompt()
        commodity = sell.commodity
        adj_txns = hledger2txn(self.file, commodity, self.no_desc)

        if self.avg_cost:
            txn_print = avg.avg_sell(
                txns=adj_txns,
                date=sell.date,
                qtty=sell.quantity,
                cur=sell.commodity,
                cash_account=sell.cash_account,
                revenue_account=sell.revenue_account,
                comm_account=sell.commodity_account,
                value=sell.value,
                check=self.check,
            )
        else:
            sell_fifo = fifo.get_sell_lots(
                lots=adj_txns,
                sell_date=sell.date,
                sell_qtty=sell.quantity,
                check=self.check,
            )
            txn_print = fifo.txn2hl(
                txns=sell_fifo,
                date=sell.date,
                cur=sell.commodity,
                cash_account=sell.cash_account,
                revenue_account=sell.revenue_account,
                value=sell.value,
            )

        return txn_print
